document.addEventListener('DOMContentLoaded', function () {
  loadNeedleGroups();
  manageWhitelist();

// Handle Emoji Selection Save
  const saveButton = document.getElementById('saveNeedleSelection');
  saveButton.addEventListener('click', () => {
    const turnEmoji = document.getElementById('turnEmoji').value || "👉"; // Default emoji
    const goldEmoji = document.getElementById('goldEmoji').value || "💛"; // Default emoji

    // Save the selected emojis to chrome.storage.sync
    chrome.storage.sync.set({ turnEmoji, goldEmoji }, () => {
      console.log('Saved custom emojis:', turnEmoji, goldEmoji);

      // Save selected needle groups
      const selectedNeedleGroups = [];
      const checkboxes = document.querySelectorAll('input[name="needleGroup"]:checked');
      checkboxes.forEach((checkbox) => {
        selectedNeedleGroups.push(checkbox.value);
      });
      chrome.storage.sync.set({ selectedNeedleGroups }, () => {
        console.log('Saved selected needle groups:', selectedNeedleGroups);

        // After saving settings, reload the active tab
        reloadActiveTab();
      });
    });
  });

  function reloadActiveTab() {
    if (confirm("Your changes have been saved. The page will now reload to apply the new settings. Do you want to proceed?")) {
      chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
        if (tabs.length > 0) {
          let tabId = tabs[0].id;
          chrome.tabs.reload(tabId, {}, function () {
            if (chrome.runtime.lastError) {
              console.error("Error reloading tab:", chrome.runtime.lastError);
            } else {
              console.log("Active tab reloaded.");
            }
          });
        } else {
          console.error("No active tab found.");
        }
      });
    } else {
      alert("Changes will be applied the next time you reload the page.");
    }
  }

  // Load the available needle groups from cached needles.json in chrome.storage.local
  function loadNeedleGroups() {
    chrome.storage.local.get('needlesData', function (data) {
      if (data && data.needlesData && data.needlesData.data && data.needlesData.data.needleGroups) {
        const needleGroups = data.needlesData.data.needleGroups;
        const needleContainer = document.getElementById('needleContainer');

        // Clear existing content
        needleContainer.innerHTML = '';

        // Dynamically create checkboxes for each needle group with display name and emoji
        Object.keys(needleGroups).forEach(groupKey => {
          const group = needleGroups[groupKey]; // Correctly access the group object
          if (group && group.displayName && group.emoji) {
            const div = document.createElement('div');
            div.className = 'form-check';  // Bootstrap class for checkbox

            const label = document.createElement('label');
            label.className = 'form-check-label';

            const checkbox = document.createElement('input');
            checkbox.type = 'checkbox';
            checkbox.name = 'needleGroup';
            checkbox.value = groupKey; // Save the key as the value
            checkbox.className = 'form-check-input';

            // Set the display name and group emoji in the label
            label.appendChild(checkbox);
            label.appendChild(document.createTextNode(` ${group.displayName} (${group.emoji})`));
            div.appendChild(label);

            needleContainer.appendChild(div);
          } else {
            console.error('Error accessing needle group data for key:', groupKey);
          }
        });

        // After the checkboxes are created, load saved selections and pre-check them
        loadSavedSelections();
      } else {
        console.error('No needle groups found in local storage.');
      }
    });
  }

  // Load saved selections and pre-check the checkboxes
  function loadSavedSelections() {
    chrome.storage.sync.get('selectedNeedleGroups', function (data) {
      let selectedNeedleGroups = data.selectedNeedleGroups;

      // If no selection exists, set 'demo' as the default selected group
      if (!selectedNeedleGroups) {
        selectedNeedleGroups = ['demo'];  // Default selection
        chrome.storage.sync.set({ selectedNeedleGroups }, () => {
          console.log('Default selected needle groups set:', selectedNeedleGroups);
        });
      }

      // Pre-select the checkboxes based on saved needle groups
      selectedNeedleGroups.forEach(groupKey => {
        const checkbox = document.querySelector(`input[value="${groupKey}"]`);
        if (checkbox) {
          checkbox.checked = true;
        }
      });
    });
  }

  // Force reload function that clears the cache and reloads the latest local files
  const forceReloadButton = document.getElementById('forceReload');
  forceReloadButton.addEventListener('click', () => {
    forceReloadCache();
  });

  function forceReloadCache() {
    console.log("Forcing reload of local files, clearing cache...");

    // Clear the cached data
    chrome.storage.local.remove(['needlesData', 'keywordsData', 'lastModified'], () => {
      console.log("Cache cleared. Reloading latest local files...");

      // Fetch the packaged files and apply them
      fetch(chrome.runtime.getURL('rules/needles.json'))
          .then(response => response.json())
          .then(needlesData => {
            // Cache the file locally for future use
            const lastModified = Date.now();  // Use the current time as the last modified timestamp
            chrome.storage.local.set({ needlesData, lastModified }, () => {
              console.log('Packaged files cached locally after force reload');
              alert("Cache reloaded successfully.");  // Show confirmation to the user
              // Reload the needle groups after reloading the cache
              loadNeedleGroups();
            });
          })
          .catch(err => console.error('Error loading needles.json during force reload:', err));
    });
  }

  // Function to manage the whitelist
  function manageWhitelist() {
    const whitelistStatus = document.getElementById('whitelistStatus');
    const toggleWhitelistButton = document.getElementById('toggleWhitelistButton');

    // Get the current tab's URL
    chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
      if (tabs.length === 0) {
        console.error('No active tab found.');
        return;
      }

      const tab = tabs[0];
      const url = new URL(tab.url);
      const currentDomain = normalizeDomain(url.hostname);

      // Get the whitelist from storage
      chrome.storage.sync.get('whitelist', function (data) {
        let whitelist = data.whitelist || [];

        if (whitelist.includes(currentDomain)) {
          // Domain is whitelisted
          whitelistStatus.textContent = `The domain "${currentDomain}" is whitelisted.`;
          toggleWhitelistButton.textContent = 'Remove from Whitelist';
        } else {
          // Domain is not whitelisted
          whitelistStatus.textContent = `The domain "${currentDomain}" is not whitelisted.`;
          toggleWhitelistButton.textContent = 'Add to Whitelist';
        }

        // Add click event listener to the button
        toggleWhitelistButton.addEventListener('click', function () {
          if (whitelist.includes(currentDomain)) {
            // Remove the domain from the whitelist
            whitelist = whitelist.filter(domain => domain !== currentDomain);
          } else {
            // Add the domain to the whitelist
            whitelist.push(currentDomain);
          }

          // Save the updated whitelist
          chrome.storage.sync.set({ whitelist }, function () {
            console.log('Whitelist updated:', whitelist);

            // Reload the active tab to apply changes
            chrome.tabs.reload(tab.id, {}, function () {
              if (chrome.runtime.lastError) {
                console.error('Error reloading tab:', chrome.runtime.lastError);
              } else {
                console.log('Active tab reloaded.');
                // Update the UI
                if (whitelist.includes(currentDomain)) {
                  whitelistStatus.textContent = `The domain "${currentDomain}" is whitelisted.`;
                  toggleWhitelistButton.textContent = 'Remove from Whitelist';
                } else {
                  whitelistStatus.textContent = `The domain "${currentDomain}" is not whitelisted.`;
                  toggleWhitelistButton.textContent = 'Add to Whitelist';
                }
              }
            });
          });
        });
      });
    });
  }

// Function to normalize the domain (remove 'www.' prefix)
  function normalizeDomain(hostname) {
    return hostname.replace(/^www\./, '');
  }
});