// background.js

import { loadLocalVersionAndApplyFilters } from './cacheManager.js';
import { filterOrReplaceContent } from './contentFilter.js';

// Function to load data (only needles and keywords now)
function loadData(callback) {
    console.log("loadData function called");
    Promise.all([
        fetch(chrome.runtime.getURL('rules/needles.json')).then(res => res.json()),
        fetch(chrome.runtime.getURL('rules/keywords.json')).then(res => res.json())
    ]).then(([needlesData, keywordsData]) => {
        chrome.storage.local.set({ needlesData, keywordsData }, function () {
            console.log("Data loaded and saved in local storage.");
            if (callback) callback();
        });
    }).catch(error => console.error("Error loading data:", error));
}

// Event listeners
chrome.runtime.onInstalled.addListener(() => {
    console.log("Extension installed or updated.");
    loadData();
});

chrome.runtime.onStartup.addListener(() => {
    console.log("Extension started.");
    loadData();
});

chrome.webNavigation.onHistoryStateUpdated.addListener(function(details) {
    console.log('Navigation event detected:', details);
    chrome.scripting.executeScript({
        target: { tabId: details.tabId },
        files: ['scripts/content.js']
    });
});

// Message listener
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
    if (request.message === 'apply_filters') {
        console.log("Received apply_filters message in background script");

        // Ensure data is loaded
        chrome.storage.local.get(['needlesData', 'keywordsData'], function (data) {
            if (!data.needlesData || !data.keywordsData) {
                console.log("Data not found in local storage, loading data...");
                loadData(() => {
                    // After loading, retrieve it again
                    chrome.storage.local.get(['needlesData', 'keywordsData'], function (data) {
                        if (data.needlesData && data.keywordsData) {
                            applyFilters(request, data.needlesData, data.keywordsData, sendResponse);
                        } else {
                            sendResponse({ error: "Data still not found after loading." });
                        }
                    });
                });
            } else if (request.message === 'reprocess_active_tab') {
                console.log("Received reprocess_active_tab message.");
                reprocessActiveTab(sendResponse);
                return true; // Indicates that sendResponse will be called asynchronously
            } else {
                // Data is available, proceed to apply filters
                applyFilters(request, data.needlesData, data.keywordsData, sendResponse);
            }
        });

        return true; // Indicates that sendResponse will be called asynchronously
    }
});

function applyFilters(request, needlesData, keywordsData, sendResponse) {
    chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
        if (chrome.runtime.lastError) {
            console.error("Error querying tabs:", chrome.runtime.lastError);
            sendResponse({ status: "Error querying tabs", error: chrome.runtime.lastError });
            return;
        }

        if (tabs.length === 0) {
            console.error("No active tab found.");
            sendResponse({ status: "No active tab found" });
            return;
        }

        console.log("Active tab found:", tabs[0].id);

        chrome.scripting.executeScript({
            target: { tabId: tabs[0].id },
            func: filterOrReplaceContent,  // The function to execute in the tab context
            args: [request.elements, needlesData, request.filterType, keywordsData]  // Pass the data
        }, (results) => {
            if (chrome.runtime.lastError) {
                console.error("Error executing script:", chrome.runtime.lastError);
                sendResponse({ status: "Error executing script", error: chrome.runtime.lastError });
            } else {
                console.log("Script executed successfully on tab:", tabs[0].id, results);
                sendResponse({ status: "Script executed successfully", results: results });
            }
        });
    });
}

function reprocessActiveTab(sendResponse) {
    chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
        if (tabs.length > 0) {
            let tabId = tabs[0].id;
            // Send a message to the content script to reapply filters
            chrome.tabs.sendMessage(tabId, { message: 'reapply_filters' }, function (response) {
                if (chrome.runtime.lastError) {
                    console.error("Error sending message to content script:", chrome.runtime.lastError);
                    sendResponse({ status: "Error sending message to content script", error: chrome.runtime.lastError });
                } else {
                    console.log("Content script re-applied filters.");
                    sendResponse({ status: "Content script re-applied filters" });
                }
            });
        } else {
            console.error("No active tab found.");
            sendResponse({ status: "No active tab found" });
        }
    });
}