// content.js

console.log("Content script loaded, checking if domain is whitelisted...");

// Prevent multiple instances of the content script
if (window.contentScriptInitialized) {
    console.log('Content script already initialized.');
} else {
    window.contentScriptInitialized = true;

    // Get the current domain
    const currentDomain = normalizeDomain(window.location.hostname);

    // Check if the domain is whitelisted
    chrome.storage.sync.get('whitelist', function (data) {
        const whitelist = data.whitelist || [];

        if (whitelist.includes(currentDomain)) {
            console.log(`Domain "${currentDomain}" is whitelisted. No filters will be applied.`);
            return; // Exit the script
        } else {
            console.log(`Domain "${currentDomain}" is not whitelisted. Applying filters...`);
            initializeContentScript();
        }
    });

    // Function to normalize the domain (remove 'www.' prefix)
    function normalizeDomain(hostname) {
        return hostname.replace(/^www\./, '');
    }

    // Function to initialize the content script
    function initializeContentScript() {
        applyFilters();

        // Set up a MutationObserver to detect changes in the DOM
        const observer = new MutationObserver((mutations) => {
            // Debounce to prevent excessive calls
            if (window.contentScriptDebounceTimer) {
                clearTimeout(window.contentScriptDebounceTimer);
            }
            window.contentScriptDebounceTimer = setTimeout(() => {
                console.log("DOM changed, re-applying filters...");
                applyFilters();
            }, 500); // Adjust the debounce interval as needed
        });

        // Start observing the body for changes
        observer.observe(document.body, {
            childList: true,
            subtree: true,
            characterData: false,
            attributes: false
        });

        // Detect URL changes (useful for SPAs)
        let currentUrl = location.href;

        // Function to check for URL changes
        function checkUrlChange() {
            const newUrl = location.href;
            if (currentUrl !== newUrl) {
                console.log(`URL changed from ${currentUrl} to ${newUrl}`);
                currentUrl = newUrl;
                applyFilters();
            }
        }

        // Set up an interval to check for URL changes
        setInterval(checkUrlChange, 1000); // Adjust the interval as needed

        // Optionally, listen to popstate events
        window.addEventListener('popstate', function(event) {
            console.log('popstate event detected');
            applyFilters();
        });
    }

    // Function to apply filters
    function applyFilters() {
        // Check again if the domain is whitelisted (in case of URL change)
        const currentDomain = normalizeDomain(window.location.hostname);
        chrome.storage.sync.get('whitelist', function (data) {
            const whitelist = data.whitelist || [];

            if (whitelist.includes(currentDomain)) {
                console.log(`Domain "${currentDomain}" is whitelisted. No filters will be applied.`);
                return; // Do not apply filters
            } else {
                console.log(`Applying filters on domain: ${currentDomain}`);

                // Send a message to the background script to apply filters
                chrome.runtime.sendMessage(
                    { message: 'apply_filters', elements: ['p', 'a', 'span', 'div'], filterType: 'replace' },
                    function (response) {
                        if (chrome.runtime.lastError) {
                            console.error('Error sending message to background:', chrome.runtime.lastError);
                        } else {
                            console.log('Response from background:', response);
                        }
                    }
                );
            }
        });
    }
}